/*
 * Copyright (C) 1997-2000, R3vis Corporation.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA, or visit http://www.gnu.org/copyleft/lgpl.html.
 *
 * Original Contributor:
 *   Wes Bethel, R3vis Corporation, Marin County, California
 * Additional Contributor(s):
 *
 * The OpenRM project is located at http://openrm.sourceforge.net/.
 */
/*
 * $Id: rmpipe.h,v 1.7 2001/06/03 20:55:24 wes Exp $
 * Version: $Name: OpenRM-1-4-0-FCS2 $
 * $Revision: 1.7 $
 * $Log: rmpipe.h,v $
 * Revision 1.7  2001/06/03 20:55:24  wes
 * Added declaration for rmPipeProcessingModeIsMultithreaded().
 *
 * Revision 1.6  2001/03/31 17:13:59  wes
 * v1.4.0-alpha-2 checkin
 *
 * Revision 1.5  2000/08/28 01:33:19  wes
 * Removed unused code.
 *
 * Revision 1.4  2000/05/14 23:36:12  wes
 * Added attrib to RMpipe to allow control over how the OpenGL
 * matrix stack is initialized during rendering.
 *
 * Revision 1.3  2000/04/20 17:41:46  wes
 * Minor CVS tag changes.
 *
 * Revision 1.2  2000/04/20 16:30:23  wes
 * Added copyright info.
 *
 * Revision 1.1.1.1  2000/02/28 21:29:40  wes
 * OpenRM 1.2 Checkin
 *
 * Revision 1.1.1.1  2000/02/28 17:18:48  wes
 * Initial entry - pre-RM120 release, source base for OpenRM 1.2.
 *
 */

#ifndef _rmpipe_h
#define _rmpipe_h

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Since the contents of the RMpipe are specific to the window
 * system, each of rmw.h and rmx.h contain the RMpipe definitions
 * for Win32 and X11, respectively.
 *
 * This file contains pipe-related tools that are not window-system specific.
 */

/* rmpipe.c + [rmw.c, rmx.c]: pipe routines [20] */
RM_EXPORT_API  RMpipe *rmPipeNew (void);
RM_EXPORT_API  RMenum  rmPipeDelete (RMpipe *toDelete);
RM_EXPORT_API  RMenum  rmPipeInit (const char *display, RMenum channel_format, RMpipe **pipe); /* defined in rmx.c or rmw.c */
RM_EXPORT_API  RMenum  rmPipeMakeCurrent (RMpipe *pipe);
RM_EXPORT_API  RMenum  rmPipeClose (RMpipe *toClose);
RM_EXPORT_API  RMenum  rmPipeSetChannelFormat (RMpipe *toModify, RMenum newChannelFormat);
RM_EXPORT_API  RMenum  rmPipeGetChannelFormat (const RMpipe *toQuery);
RM_EXPORT_API  RMenum  rmPipeSetInitMatrixStackMode(RMpipe *toModify, RMenum newMode);
RM_EXPORT_API  RMenum  rmPipeGetInitMatrixStackMode(const RMpipe *toQuery);
RM_EXPORT_API  RMenum  rmPipeSetRenderPassEnable (RMpipe *t, RMenum opaque3DEnable, RMenum transparent3DEnable, RMenum opaque2DEnable);
RM_EXPORT_API  RMenum  rmPipeGetRenderPassEnable (const RMpipe *t, RMenum *opaque3DEnableReturn, RMenum *transparent3DEnableReturn, RMenum *opaque2DEnableReturn);
RM_EXPORT_API  RMenum  rmPipeSetWindowSize (RMpipe *toModify, int newWidth, int newHeight);
RM_EXPORT_API  RMenum  rmPipeGetWindowSize (const RMpipe *toQuery, int *widthReturn, int *heightReturn);
RM_EXPORT_API  RMenum  rmPipeSetSwapbuffersFunc (RMpipe *toModify, void (*newFunc)(RMpipe *));
RM_EXPORT_API  RMenum  rmPipeSetPostRenderFunc (RMpipe *toModify, void (*postRenderFunc)(const RMimage *, RMenum));
RM_EXPORT_API  RMenum  rmPipeSetPostRenderDepthFunc (RMpipe *toModify, void (*postRenderDepthFunc)(const RMimage *, RMenum));
RM_EXPORT_API  RMenum  rmPipeSetPostRenderBarrierFunc (RMpipe *toModify, void (*barrierFunc)(RMpipe *));
RM_EXPORT_API  RMenum  rmPipeGetProcessingMode(const RMpipe *toQuery);
RM_EXPORT_API  RMenum  rmPipeSetProcessingMode(RMpipe *toModify, RMenum newMode);
RM_EXPORT_API  RMenum  rmPipeProcessingModeIsMultithreaded(const RMpipe *toQuery);

/* to be deprecated */
RM_EXPORT_API  RMpipe *rmPipeGetCurrent (void);


#ifdef __cplusplus
}
#endif

#endif /* _rmpipe_h */
/* EOF */
