/*
 * Copyright (C) 1997-2000, R3vis Corporation.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA, or visit http://www.gnu.org/copyleft/lgpl.html.
 *
 * Original Contributor:
 *   Wes Bethel, R3vis Corporation, Marin County, California
 * Additional Contributor(s):
 *
 * The OpenRM project is located at http://openrm.sourceforge.net/.
 */

/*
 * $Id: rmtypes.h,v 1.8 2001/07/15 17:21:20 wes Exp $
 * Version: $Name: OpenRM-1-4-0-FCS2 $
 * $Revision: 1.8 $
 * $Log: rmtypes.h,v $
 * Revision 1.8  2001/07/15 17:21:20  wes
 * Removed dead code.
 *
 * Revision 1.7  2001/03/31 17:13:59  wes
 * v1.4.0-alpha-2 checkin
 *
 * Revision 1.6  2000/12/03 23:20:22  wes
 * Thread-safety mods.
 *
 * Revision 1.5  2000/09/02 19:26:21  wes
 * Updated comments inside code.
 *
 * Revision 1.4  2000/05/17 14:31:19  wes
 * Added rmPolys() (thanks to Matt and Todd at VRCO).
 *
 * Revision 1.3  2000/04/20 17:41:46  wes
 * Minor CVS tag changes.
 *
 * Revision 1.2  2000/04/20 16:30:24  wes
 * Added copyright info.
 *
 * Revision 1.1.1.1  2000/02/28 21:29:40  wes
 * OpenRM 1.2 Checkin
 *
 * Revision 1.1.1.1  2000/02/28 17:18:48  wes
 * Initial entry - pre-RM120 release, source base for OpenRM 1.2.
 *
 */

#ifndef _rmtypes_h
#define _rmtypes_h

#include <rm/rmthread.h>
#include <rm/rmcmpmgr.h>

/* constants and limits */
#define MAX_VIS_CMAP_ENTRIES	256
#define RMV_DEFAULT_MAP_SIZE	256
#define OBJECT_NAME_LENGTH	64

#ifdef __cplusplus
extern "C" {
#endif

/* color types [4] */
typedef struct RMcolor1D
{
    float lum;
} RMcolor1D;

typedef struct RMcolor2D
{
    float lum, alpha;
} RMcolor2D;

typedef struct RMcolor3D
{
    float r, g, b;
} RMcolor3D;

typedef struct RMcolor4D
{
    float r, g, b, a;
} RMcolor4D;

/* vertex types [3] */
typedef struct RMvertex2D
{
    float x, y;
} RMvertex2D;

typedef struct RMvertex3D
{
    float x, y, z;
} RMvertex3D;

typedef struct RMvertex4D
{
    float x, y, z, w;
} RMvertex4D;

typedef struct RMmatrix
{
    float m[4][4];
} RMmatrix;

/* 32 bit cache key */
typedef unsigned int RMcacheKey;

#if 0
/* 64 bit cache key */
typedef unsigned long RMcacheKey;
#endif

typedef struct RMinternalMarker2D
{
    int         npts;
    int         gl_begin_flag;
    RMvertex2D *vlist;
} RMinternalMarker2D;

typedef struct RMvisMap
{
    int   nentries;
    float r[MAX_VIS_CMAP_ENTRIES];
    float g[MAX_VIS_CMAP_ENTRIES];
    float b[MAX_VIS_CMAP_ENTRIES];
    float a[MAX_VIS_CMAP_ENTRIES];
    float transfer_min, transfer_max;
} RMvisMap;

typedef struct RMbitmap
{
    int            w, h;
    int            bytes_per_scanline;
    int            pbsize;
    unsigned char *pixeldata;
} RMbitmap;

typedef struct RMimage
{
    /* configuration */
    int    ndims;		/* 2 or 3, set implicitly by user */
    int    w, h, d;		/* set by user */
    RMenum image_format;	/* set by user: RM_IMAGE_RGB, etc. */
    RMenum image_type;		/* set by user: RM_IMAGE_BYTE, etc. */
    float  xzoom, yzoom;	/* for 2d images only, user supplied, default 1.0, 1.0 */

    /* pixel data */
    RMenum copy_flag;		 /* set by user */
    void  *pixeldata;		 /* set by user */
    void (*appfreefunc)(void *); /* set by user */

    /* colormap */
    RMvisMap *vismap;		 /* set by user, default NULL */

    /* scale & bias */
    float scale, bias;

    /* internals, private */
    int          bytes_per_component; /* computed by RM */
    int          bytes_per_scanline;  /* computed by RM */
    int          elements;	      /* computed by RM */
#if 0
    /* display list stuff removed 10/5/2000 */
    int          d1;
    RMenum       dirty_flag;
#endif
    unsigned int pbsize;

    /* internal, component manager */
    int          compListIndx;
    
    /* internal, context cache key */
    RMcacheKey   cacheKey;
} RMimage;

typedef struct RMcamera2D
{
    float xmin, ymin, xmax, ymax;  /* defines extents of camera (like a viewport, but in R2 world coords) */
    float aspect_ratio;
} RMcamera2D;

typedef struct _RMStereoCameraControls
{
    void (*preframe_leftchannel_func)();
    void (*preframe_rightchannel_func)();
} _RMStereoCameraControls;

typedef struct RMcamera3D
{
    /* camera points */
    RMvertex3D eye;
    RMvertex3D at;
    RMvertex3D up;

    float  hither, yon; 		/* near & far */
    float  fov;				/* field of view angle in degrees in the Y direction */
    float  aspect;			/* aspect ratio (width / height) --> field of view angle in X direction */
    RMenum projection;			/* may be RM_PROJECTION_PERSPECTIVE or RM_PROJECTION_ORTHOGRAPHIC */
    RMenum isStereo;			/* flag for stereo (RM_TRUE or RM_FALSE) */
    float  degrees_eye_separation;	/* for stereo cameras only */
    float  focalDistance;		/* for stereo cameras only */
} RMcamera3D;

typedef struct RMtextProps
{
    int    fontEnum;		/* one of RM_FONT_SERIF,RM_FONT_SANS, RM_FONT_MONO, RM_FONT_SYMBOL */
    int    italicEnum;		/* RM_TRUE or RM_FALSE */
    int    boldEnum;		/* RM_TRUE or RM_FALSE */
    int    sizeEnum; 		/* enum of RM_FONT_XXS, RM_FONT_XS, RM_FONT_S, RM_FONT_M, RM_FONT_L, RM_FONT_XL, RM_FONT_XXL */
    RMenum hJustify, vJustify;
    int    listbase;		/* OpenGL dislay list base (computed by RM at runtime) */
    int    listoffset;		/* OpenGL display list stuff (computed by RM at runtime) */
    void  *fontinfo;

    int    compListIndx;
} RMtextProps;

typedef struct _bounding_box
{
    RMvertex3D min;
    RMvertex3D max;
} _bounding_box;

typedef struct RMclipPlane
{
    RMenum     enabled;		/* RM_TRUE, RM_FALSE */
    RMvertex3D point;
    RMvertex3D normal;		/* (point, normal) parametrization */	
   float       a, b, c, d;	/* plane equation */
} RMclipPlane;

typedef struct RMlight
{
   /* individual light ala OpenGL Light (see glLight) */
    RMenum     ltype;			/* RM_LIGHT_POINT, RM_LIGHT_DIRECTIONAL, RM_SPOT */
    RMcolor4D  ambientLightColor;
    RMcolor4D  diffuseLightColor;
    RMcolor4D  specularLightColor;
    RMvertex3D lightXYZ;		/* position for point and spot lights, direction for directional light sources */
    float      spotCutoff;		/* [0.0, 90.0] degrees */
    float      spotExponent;		/* default = 0.0, higher numbers produce more focused spots */
    RMvertex3D spotDirection;
    float      constantAttenuation;
    float      linearAttenuation;
    float      quadraticAttenuation;
    RMenum     enabled;			/* RM_TRUE or RM_FALSE */
} RMlight;

typedef struct RMlightModel
{
    /* light model parms ala OpenGL Light Model (see glLightModel) */
    RMcolor4D globalAmbient;
    GLenum    twoSideEnable;
    GLenum    localViewerEnable;
} RMlightModel;

typedef struct RMtexture
{
    RMimage   *images[RM_MAX_MIPMAPS];

    int       dims;			/* 1, 2 or 3d texture */
    int       nmipmaps;			/* number of mipmaps associated w/this texture */
#if 0
    GLuint    tname;			/* generated by glGenTextures */
#endif

    /* flags for OpenGL */
    GLenum    mag_filter_mode;		/* for GL_TEXTURE_MAG_FILTER */
    GLenum    min_filter_mode;		/* for GL_TEXTURE_MIN_FILTER */
    GLenum    wrap_mode;		/* applied to each of u, v, w */

    /* OpenGL has extra options for texel storage modes */
    GLenum    *oglTexelFormat;		/* GL_RGBA, GL_RGB, etc. default is GL_RGBA */

    /* environment modes */
    GLenum     envMode;			/* see man glTexEnv */
    RMcolor4D *blendColor;		/* see man glTexEnv */

    int        borderWidth;		/* texture border */

    /* residency status currently not used. */
    RMenum     residency_status;	/* internal: RM_TRUE or RM_FALSE */

    /* for the component manager */
    int        compListIndx;
    
    /* internal, context cache keys for texture IDs and texture data */
    RMcacheKey  cacheKeyID;
    RMcacheKey  cacheKeyData;
} RMtexture;

typedef struct RMfog
{
    GLenum    fogMode;		/* GL_LINEAR, GL_EXP or GL_EXP2, def GL_LINEAR */
    GLfloat   fogDensity;	/* default = 1.0 */
    GLfloat   fogStart;		/* default is 0.0 */
    GLfloat   fogEnd;		/* default is 1 */
    RMcolor4D fogColor;		/* default is 0,0,0,0 */
} RMfog;

typedef struct RMstate
{
    RMmatrix      model;		/* model transformations */
    RMmatrix      view;			/* viewing transformations */
    RMmatrix      modelView;		/* combined model & view (OGL style) */
    RMmatrix      projection;		/* projection matrix */
    RMmatrix      composite;	        /* model*view*projection */
    RMmatrix      pick;			/* pick matrix */
    RMmatrix      textureMatrix;	/* texture matrix */
    RMmatrix      projection_inverse;	/* inverse of the concatenated view & projection matrices  (?) */
    RMmatrix      vpm;			/* viewport matrix - from NDC to window */
    float         vp[4];		/* (x, y) VP location + (w, h) VP width & height */
    float         aspect_ratio; 
    float         focal_length; 
    int           w, h;			/* window width & height */
    RMenum        rendermode;
    RMenum        renderpass;
    RMenum        which_channel;	/* used */
    RMtexture    *texture;		/* current texture */
    int           texture_mode;		/* if current texture, GL_TEXTURE_2D, etc. */
    int           attrib_stack_depth;	/* current attribute stack depth */
    
    /* material properties */
    RMcolor4D     ambient;
    RMcolor4D     diffuse;
    RMcolor4D     specular;
    RMcolor4D     emissive;
    RMcolor4D     unlit_color;
    float         specular_exponent;
    float         opacity;

    /* line & point style */
    float         pointsize;
    RMenum        linewidth;
    RMenum        linestyle;

    /* render mode parms */
    RMenum        shademodel;
    
    /* polygon face properties */
    RMenum        poly_mode_face;
    RMenum        poly_mode_drawstyle;
    RMenum        cull_mode;		/* cull faces? [RM_CULL_NONE, RM_CULL_FRONT, RM_CULL_BACK, RM_CULL_FRONT_AND_BACK] */
    RMenum        front_face;   	/* vertex winding? [RM_CCW, RM_CW] */

    /* clip planes */
    RMclipPlane  *cp0;
    RMclipPlane  *cp1;
    RMclipPlane  *cp2;
    RMclipPlane  *cp3;
    RMclipPlane  *cp4;
    RMclipPlane  *cp5;
    
    /* lighting */
    RMlightModel *lmodel;
    RMlight      *lightSources[RM_MAX_LIGHTS];

    /* text */
    RMtextProps  *textProps;

    int          frameNumber;
} RMstate;

typedef struct _surface_properties
{
    RMcolor4D *ambient_color;
    RMcolor4D *diffuse_color;
    RMcolor4D *specular_color;
    RMcolor4D *unlit_color;
    float     *specular_exponent;
    float     *opacity;
} _surface_properties;
   
typedef struct _rendermode_properties
{
    RMenum *shademodel;			/* shade? [RM_SHADER_SMOOTH, RM_SHADER_FLAT, RM_SHADER_NOLIGHT] */
    RMenum *poly_mode_face;
    RMenum *poly_mode_drawstyle;
    RMenum *cull_mode;			/* cull faces? [RM_CULL_NONE, RM_CULL_FRONT, RM_CULL_BACK, RM_CULL_FRONT_AND_BACK] */
    RMenum *front_face;			/* vertex winding? [RM_CCW, RM_CW] */
    float  *pointsize;
    RMenum *linewidth;
    RMenum *linestyle;
    RMenum *normalizeNormals;
} _rendermode_properties;

typedef struct internals_RMsceneParms
{
    float        *viewport; 		 /* array of length 4: (x, y, w, h) in NDC [0..1], scales to window size */
/*    RMcolor4D    *background_color; moved */
/*    RMimage      *background_image_tile; moved */
    RMcamera3D   *camera3d;
    RMcamera2D   *camera2d;
    RMtexture    *texture;
/*    float        *depth_value; moved */
/*    RMimage      *depth_image; moved */
    RMclipPlane  *cp0;
    RMclipPlane  *cp1;
    RMclipPlane  *cp2;
    RMclipPlane  *cp3;
    RMclipPlane  *cp4;
    RMclipPlane  *cp5;
    RMlightModel *lmodel;
    RMlight      *lightSources[RM_MAX_LIGHTS];
    RMtextProps  *textProps;
    RMfog        *fog;
} internals_RMsceneParms;

typedef struct internals_RMfbClear
{
    RMcolor4D    *bgColor; 
    RMimage      *bgImageTile;
    
    float        *depthValue; 
    RMimage      *depthImage;
} internals_RMfbClear;

typedef struct internals_RMtransformationStruct
{
    /*
     * transformations are applied in this order (left multiplication!):
     *
     * Pre * -C * S * R * S2 * C * T * Post
     */
    RMmatrix   pre;		/* applied before any other transforms */
    RMmatrix   s;		/* pre-rotation scale */
    RMmatrix   r;		/* rotation */
    RMmatrix   s2;		/* post-rotation scale */
    RMvertex3D translate;	/* translation vector */
    RMmatrix   post;		/* applied after other transforms */
    RMenum     transform_mode;  /* RM_TRANSFORM_GEOMETRY, RM_TRANSFORM_TEXTURE, RM_TRANSFORM_CAMERA, RM_TRANSFORM_IGNORE */
} internals_RMtransformationStruct;

typedef struct _object_info
{
    char   name[OBJECT_NAME_LENGTH];

    /* node scheduling for rendering in a multipass/multichannel rendering environment */
    RMenum posted;		/* controls whether or not is rendered */
    RMenum rpass_vdims;		/* 2D or 3D coords? */
    RMenum rpass_opacity;	/* RM_RENDERPASS_OPAQUE, RM_RENDERPASS_TRANSPARENT */
    RMenum channel;		/* RM_LEFT_CHANNEL, RM_RIGHT_CHANNEL, RM_ALL_CHANNELS */
} _object_info;

typedef struct RMnode
{
    /* scene graph topology */
    struct RMnode  *parent;
    int             nchildren;
    struct RMnode **children;

    /* here's the geometry */
    int             nprims;
    void          **prims;

    /* properties */
    _surface_properties    *sprops;
    _rendermode_properties *rprops;
    internals_RMsceneParms *scene_parms;

    /* fb clear control */
    internals_RMfbClear    *fbClear;

    /* bounding box info */
    _bounding_box           bbox;
    RMvertex3D              center;

    /* transformation info */
    internals_RMtransformationStruct *transforms;

    /* misc scheduling info about the object */
    _object_info object_info;

    /* callbacks */
    int (*viewSwitchCallback) (const struct RMnode *, const RMstate *);
    int (*viewPretraverseCallback)    (const struct RMnode *, const RMstate *);
    int (*viewPosttraverseCallback)   (const struct RMnode *, const RMstate *);

    int (*renderPretraverseCallback)    (const struct RMnode *, const RMstate *);
    int (*renderPosttraverseCallback)   (const struct RMnode *, const RMstate *);

    /* client data stuff - not multistage safe! (1/2001) */
    void  *clientData;
    void (*clientDataFreeFunc) (struct RMnode *, void *);

    int  refcount;

    /* attribute mask */
    GLuint   attribMask;

    /* optional mutex */
    RMmutex *nodeMutex;
    
    /* internal, component manager */
    int          compListIndx;
} RMnode;

typedef struct RMfontRegistry /* PRIVATE */
{
    int   initialized;
    int   refcount;
    int   listbase;
    int   listoffset;
    void *fontinfo;
} RMfontRegistry;

/*
 * context cache structure - used internally by RM to hold OpenGL
 * display list IDs, texture IDs, etc. 
 */
typedef struct RMcontextCache
{
    /* for RMprimitives */
    GLuint      *primDisplayListIDs;
    RMcacheKey  *primCacheKeys;

    /* for RMimages */
    GLuint      *imgDisplayListIDs;
    RMcacheKey  *imgCacheKeys;

    /* for RMtextures */
    GLuint      *textureIDs;
    RMcacheKey  *textureIDCacheKeys; /* textureID management */
    RMcacheKey  *textureDataCacheKeys; /* texture image data status */

    /* need to add this stuff VVVV  */
    
    /* RM-built display lists for quadrics objects */
    GLuint      *sphereIDs;
    GLuint      *coneIDs, *flipConeIDs;
    GLuint      *cylinderIDs, *flipCylinderIDs;
    
    /* pre-rendered bitmap font glyph display list IDs  */
    RMfontRegistry *pipeFontRegistry;
    
} RMcontextCache;

#ifdef RM_X
typedef struct RMpipe
{
    Display     *xdisplay;
    XVisualInfo *xvisual;
    Window       xdrawable;
    GLXContext   glxcontext;
    Colormap     xcolormap;

    RMenum       processingMode;
    
    RMenum       channel_format;
    int          xwindow_width, xwindow_height;
    int          xflags;
    RMenum       opaque3DEnable, transparent3DEnable, opaque2DEnable;
    RMenum       initMatrixStack;	/* RM_TRUE or RM_FALSE */
    int          frameNumber;

    /* render function */
    void (*channel_render_func)(RMnode *t, struct RMpipe *p);

    /*
     * optional app callback invoked after rendering, but before
     * the "swapbuffers" call. can be used to perform synchronization
     * between multiple rendering engines.
     */
    void (*postRenderBarrierFunc)(struct RMpipe *p);

    /* color and depth buffer acquisition, swap buffer function */
    void (*postrenderfunc)(const RMimage *, RMenum whichBufferEnum);
    void (*postrender_depthbufferfunc)(const RMimage *, RMenum whichBufferEnum);
    void (*swapbuffunc)(struct RMpipe *);

    char   name[32];		/* temp hack */

    /* the context cache */
    RMcontextCache *cache;

    /* multistage rendering buffers */
    void *displayLists;

    /* multistage process/thread control structure */
    void *mtControl;
    
} RMpipe;
#endif

#ifdef RM_WIN
typedef struct RMpipe
{
    RMenum channel_format;
    int    xwindow_width, xwindow_height;
    int    xflags;
    RMenum opaque3DEnable, transparent3DEnable, opaque2DEnable;
    RMenum initMatrixStack;	/* RM_TRUE or RM_FALSE */
    int    frameNumber;
    
    RMenum       processingMode;
    
    /* render function */
    void (*channel_render_func)(RMnode *t, struct RMpipe *p); 

    /*
     * optional app callback invoked after rendering, but before
     * the "swapbuffers" call. can be used to perform synchronization
     * between multiple rendering engines.
     */
    void (*postRenderBarrierFunc)(struct RMpipe *p);
    
    /* color and depth buffer acquisition */
    void (*postrenderfunc)(RMimage *, int rm_buff_enum);
    void (*postrender_depthbufferfunc)(RMimage *, int rm_buff_enum);
    
    void (*swapbuffunc)(struct RMpipe *);

    HDC    hdc;
    HWND   hwnd;
    HGLRC  hRC;

    /* the context cache */
    RMcontextCache *cache;
    
    /* multistage rendering buffers */
    void *displayLists;
    
    /* multistage process/thread control structure */
    void *mtControl;
} RMpipe;
#endif

#define P_OGLPRIMPARMLIST() (struct RMprimitive *p, RMnode *r, RMstate *s, RMpipe *renderPipe)

typedef struct RMprimitiveDataBlob
{
    void  *ptr;				/* pointer to the stuff */
    RMenum copyflag;
    int    nthings;			/* how many things there are */
    int    blobtype;			/* what kind of blob? */
    int    stride;			/* size of individual thing */
    int    veclen;			/* vector length of data supplied */
    void (*appfreefunc)(void *);	/* the app-supplied free function for when the copy-flag is set to RM_DONT_COPY_DATA */
} RMprimitiveDataBlob;

typedef struct RMprimitive
{
    RMenum               type;
    RMprimitiveDataBlob *blobs;

    /* methods, data, flags */
    void (*renderfunc)  P_OGLPRIMPARMLIST();	/* RMprimitive stores a pointer to the primitive draw method */
    void  *clientData;
    void (*clientDataFreeFunc) (struct RMprimitive *p, void *cd);
    void  *p1;					/* stuff that doesn't really fit into blobs */

    /* misc stuff - usage depends upon app */
    unsigned int flags1;
    unsigned int model_flag;
    int          display_list_enable;

    /* old stuff for compatibility */
    int    utility;

    /* internal, component manager */
    int          compListIndx;

    /* internal, context cache key */
    RMcacheKey   cacheKey;
} RMprimitive;

#define OGLPRIMPARMLIST() (RMprimitive *p, RMnode *r, RMstate *s, RMpipe *renderPipe)

/* RM-supplied primitive rendering functions */
void rmTrianglesDisjoint	OGLPRIMPARMLIST();
void rmTrianglesConnected	OGLPRIMPARMLIST();
void rmLineStrip		OGLPRIMPARMLIST();
void rmLinesDisjoint		OGLPRIMPARMLIST();
void rmQuadmesh			OGLPRIMPARMLIST();
void rmQuads			OGLPRIMPARMLIST();
void rmText			OGLPRIMPARMLIST();
void rmSpheres			OGLPRIMPARMLIST();
void rmPoints			OGLPRIMPARMLIST();
void rmCones			OGLPRIMPARMLIST();
void rmCylinders		OGLPRIMPARMLIST();
void rmIndexedText		OGLPRIMPARMLIST();
void rmMarkers2D	     	OGLPRIMPARMLIST();
void rmBox3d			OGLPRIMPARMLIST();
void rmBox3dWire	    	OGLPRIMPARMLIST();
void rmCircle2d			OGLPRIMPARMLIST();
void rmBox2d			OGLPRIMPARMLIST();
void rmEllipse2d		OGLPRIMPARMLIST();
void rmSprite			OGLPRIMPARMLIST();
void rmBitmap			OGLPRIMPARMLIST();
void rmIndexedBitmap		OGLPRIMPARMLIST();
void rmTriangleFan		OGLPRIMPARMLIST();
void rmIndexedTriangleFan	OGLPRIMPARMLIST();
void rmOctmesh			OGLPRIMPARMLIST();
void rmPolys			OGLPRIMPARMLIST();

/* structure returned when picking */
typedef struct RMpick
{
    RMnode      *node;
    float        zval;		/* NDC z-value */
    unsigned int index;		/* object index */
    unsigned int prim_index;	/* primitive index */
} RMpick;

#ifdef RM_X
#include "rmx.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /* _rmtypes_h */
/* EOF */
